﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

using View;
using System.Diagnostics;

namespace Model
{
    class Warrior : Unit
    {
        public bool Alive { get; set; }
        public int Hp { get; set; }
        private int str;
        private int intelligence;
        private int skill;
        public int Speed { get; set; }
        public int Def { get; set; }
        public int Res { get; set; }
        public int Level { get; set; }
        private readonly int movability = 4;    // all warriors will have 4 movability
        private Weapon[] equipableWeapons;
        public Weapon equippedWeapon { get; set; }
        private Texture2D spriteImage;
        private Vector2 pixelCoordinates;
        private Tuple<int, int> position;
        private Button[] buttons = new Button[5];
        private Texture2D charInfo, charAttackInfo;
        private int currentFrame; //the current frame the sprite is on
        private int player; // which player this unit belongs to

        public Warrior(Texture2D spriteImage, Button attackButton, Button moveButton,
            Button itemButton, Button waitButton, Button confirmButton, Texture2D charInfo, Texture2D charAttackInfo, Vector2 coordinates, int player)
        {
            this.spriteImage = spriteImage;
            buttons[0] = attackButton;
            buttons[1] = moveButton;
            buttons[2] = itemButton;
            buttons[3] = waitButton;
            buttons[4] = confirmButton;
            this.charInfo = charInfo;
            this.charAttackInfo = charAttackInfo;
            this.player = player;
            pixelCoordinates = coordinates;
            int positionX = (int)Math.Round(coordinates.X / 32);
            int positionY = (int)Math.Round(coordinates.Y / 32);
            position = new Tuple<int, int>(positionX, positionY);
            currentFrame = 1;
            setButtonCoordinates(pixelCoordinates);
            setInitialStats(); //sets initial warrior stats
        }

        //sets initial unit stats
        public void setInitialStats()
        {
            Alive = true;
            Level = 1;
            Hp = 10;
            Str = 7;
            Int = 1;
            Skill = 6;
            Speed = 8;
            Def = 8;
            Res = 4;
        }

        //sets and gets unit effective str
        public int Str
        {
            get
            {
                return str + equippedWeapon.modStr;
            }
            set
            {
                str = value;
            }
        }

        //sets and gets unit effective int
        public int Int
        {
            get
            {
                return intelligence + equippedWeapon.modInt;
            }
            set
            {
                intelligence = value;
            }
        }

        //sets and gets unit effective skill
        public int Skill
        {
            get
            {
                return skill + equippedWeapon.modSkill;
            }
            set
            {
                skill = value;
            }
        }

        public int getMovability()
        {
            return movability;
        }

        //returns each stat as an index for incrementing purposes
        public int getStats(int i)
        {
            int[] allStats = new int[7];
            allStats[0] = Level;
            allStats[1] = Str;
            allStats[2] = Int;
            allStats[3] = Skill;
            allStats[4] = Speed;
            allStats[5] = Def;
            allStats[6] = Res;
            return allStats[i];
        }

        public Weapon[] getEquipableWeapons()
        {
            return equipableWeapons;
        }

        public UnitType getClass()
        {
            return UnitType.Warrior;
        }

        public Texture2D getSpriteImage()
        {
            return spriteImage;
        }

        public Texture2D getButtonImage(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack: // if attack clicked
                    return buttons[0].getImage();
                case ButtonType.Move: // if moved is clicked
                    return buttons[1].getImage();
                case ButtonType.Items: // if item is clicked
                    return buttons[2].getImage();
                case ButtonType.Wait: // if wait is clicked
                    return buttons[3].getImage();
                case ButtonType.AttackConfirm:
                    return buttons[4].getImage();
                default:
                    return null;
            }
        }

        // indicates whether a button has already been previously selected or not
        public bool isButtonActive(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack:
                    return buttons[0].getActive();
                case ButtonType.Move:
                    return buttons[1].getActive();
                case ButtonType.Items:
                    return buttons[2].getActive();
                case ButtonType.Wait:
                    return buttons[3].getActive();
                case ButtonType.AttackConfirm:
                    return buttons[4].getActive();

                default:
                    return false;
            }
        }

        public Texture2D getCharInfo()
        {
            return charInfo;
        }

        public Texture2D getCharAttackInfo()
        {
            return charAttackInfo;
        }

        public Tuple<int, int> Position
        {
            get
            {
                return position;
            }
            set
            {
                position = value;
                pixelCoordinates = new Vector2(value.Item1 * 32, value.Item2 * 32);
            }
        }

        public Vector2 PixelCoordinates
        {
            get
            {
                return pixelCoordinates;
            }
            set
            {
                pixelCoordinates = value;
                int positionX = (int)Math.Round(PixelCoordinates.X / 32);
                int positionY = (int)Math.Round(PixelCoordinates.Y / 32);
                position = new Tuple<int, int>(positionX, positionY);
            }
        }

        public Button[] getButtons()
        {
            return buttons;
        }

        public Button getButtonOfType(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack:
                    return buttons[0];
                case ButtonType.Move:
                    return buttons[1];
                case ButtonType.Items:
                    return buttons[2];
                case ButtonType.Wait:
                    return buttons[3];
                case ButtonType.AttackConfirm:
                    return buttons[4];

                default:
                    return null;
            }
        }

        // updates menu button positions
        public void setButtonCoordinates(Vector2 pixelCoordinates)
        {
            int increment = 0;
            foreach (Button button in buttons)
            {
                if (button.getActive())
                {
                    if (button.getButtonType() != ButtonType.AttackConfirm)
                    {
                        button.setPixelCoordinates((int)(pixelCoordinates.X + 32), (int)(pixelCoordinates.Y + (increment * 32)));
                        increment++;
                    }
                    else if (GameState.attackConfirmOpen)
                    {
                        button.setPixelCoordinates((int)(pixelCoordinates.X + 32), (int)(pixelCoordinates.Y));
                    }
                }
            }
        }

        public Rectangle getCurrentFrame() //return current frame the sprite is on
        {
            Rectangle screenBounds = new Rectangle(currentFrame * 32, 0, 32, 32);
            return screenBounds;
        }

        // animates the sprite by switching to the correct frame sequences
        public void animate(int direction)
        {

            #region Walking Down
            if (direction == 0) // for going down
            {
                if (currentFrame > 2) // if unit isnt already going down, set the sprite to default facing downwards
                {
                    currentFrame = 1;
                }
                else
                {
                    if (currentFrame == 2) // if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 0;
                    }
                    else
                    {
                        currentFrame++; // increment through walk cycle
                    }
                }
            }
            #endregion

            #region Walking Left
            else if (direction == 1) // for going left
            {
                if ((currentFrame < 3) || (currentFrame > 5)) // if unit isnt already going left, set the sprite to default facing left
                {
                    currentFrame = 4;
                }
                else
                {
                    if (currentFrame == 5) // if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 3;
                    }
                    else
                    {
                        currentFrame++; //increment walk cycle
                    }
                }
            }
            #endregion

            #region Walking Right
            else if (direction == 2) // for going right
            {
                if ((currentFrame < 6) || (currentFrame > 8)) // if unit isnt already going right, set the sprite to default facing right
                {
                    currentFrame = 7;//increment walk cycle
                }
                else
                {
                    if (currentFrame == 8)// if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 6;
                    }
                    else
                    {
                        currentFrame++;
                    }
                }
            }
            #endregion

            #region Walking Up
            else if (direction == 3) // for going up
            {
                if (currentFrame < 9) // if unit isnt already going up, set the sprite to default facing upwards
                {
                    currentFrame = 10;//increment walk cycle
                }
                else
                {
                    if (currentFrame == 11)// if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 9;//increment walk cycle
                    }
                    else
                    {
                        currentFrame++;
                    }
                }
            }
            #endregion
        }
    }
}
